<?php

namespace Drupal\easy_entity_field\Form;

use Drupal\Core\Entity\EntityConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Builds the form to delete Easy entity field entities.
 */
class EasyEntityFieldDeleteForm extends EntityConfirmFormBase {

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete %name field?', ['%name' => $this->entity->label()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    $route_name = $this->entity->toUrl('collection')->getRouteName();
    if (\Drupal::request()->get('orphaned_field')) {
      $route_name = 'entity.easy_entity_field.delete_form';
    }

    return $route_name;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->entity->delete();

    $this->messenger()->addMessage(
      $this->t('content @type: deleted @label.', [
        '@type' => $this->entity->bundle(),
        '@label' => $this->entity->label(),
      ])
    );
    
    $form_state->setRedirect($this->getCancelUrl(), ['easy_entity_field' => $this->entity->id()]);
  }

}
