<?php

namespace Drupal\easy_entity_field\Plugin\Derivative;

use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

class EasyEntityFieldLocalTask extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Creates a FieldUiLocalTask object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $this->derivatives = [];


    $entity_types = $this->entityTypeManager->getDefinitions();
    foreach ($entity_types as $entity_type) {
      if (!$entity_type instanceof ContentEntityTypeInterface) {
        continue;
      }
      $entity_type_id = $entity_type->id();
      if (($settings = \Drupal::configFactory()
          ->getEditable("easy_entity_field.settings.{$entity_type_id}"))
        && $settings->get('status')
      ) {

        $base_route_name = $entity_type->get('field_ui_base_route');

        if ($entity_type->getBundleEntityType()) {
          if ($entity_type->getProvider() == 'easy_entity') {
            $base_route_name = "entity.easy_entity_type.{$entity_type->id()}.edit_form";
          }
          else {
            if (!empty($settings->get('base_route'))) {
              $base_route_name = $settings->get('base_route');
            }
            else {
              $base_route_name = "entity.{$entity_type->getBundleEntityType()}.collection";
              $this->derivatives[$base_route_name] = [
                'route_name' => $base_route_name,
                'weight' => -10,
                'title' => $this->t('Type list'),
                'base_route' => $base_route_name,
              ];
            }
          }
        }

        if (!isset($base_route_name)) {
          continue;
        }
        $this->derivatives["easy_entity_field_default.{$entity_type_id}"] = [
          'route_name' => "entity.{$entity_type_id}.base_field",
          'weight' => 8,
          'title' => $this->t('Manage Base fields'),
          'base_route' => $base_route_name,
        ];

        $this->derivatives["easy_entity_field_edit.{$entity_type_id}"] = [
          'route_name' => "entity.{$entity_type_id}.base_field.edit",
          'title' => $this->t('Edit'),
          'base_route' => "entity.{$entity_type_id}.base_field.edit",
        ];

        $this->derivatives["easy_entity_field_storage.{$entity_type_id}"] = [
          'route_name' => "entity.{$entity_type_id}.base_field.storage",
          'title' => $this->t('Storage'),
          'base_route' => "entity.{$entity_type_id}.base_field.edit",
        ];
      }
    }

    foreach ($this->derivatives as &$entry) {
      $entry += $base_plugin_definition;
    }

    return $this->derivatives;
  }

}
